﻿// ImageEffectsWP8.cpp
#include "pch.h"
#include "ImageEffectsWP8.h"
#include <robuffer.h>
#include <ppltasks.h>


using namespace ImageEffectsWP8;
using namespace Platform;
using namespace Windows::Storage::Streams;
using namespace concurrency;
using Platform::String;

///////////////////////////////////////////////////////////////////////////////
//
//
Windows::Foundation::IAsyncAction^ ImageEffects::ApplygGrayscale(Windows::Storage::Streams::IBuffer^ buffer)
{
	if (buffer == nullptr)
		throw ref new Platform::NullReferenceException(L"buffer cannot be null");

	// To access the buffer, we need to use old-style COM to get an interface to IBufferByteAccess
	//
	IUnknown*			pUnk    = reinterpret_cast<IUnknown*>(buffer);
	IBufferByteAccess*	pAccess = NULL;
	byte*				bytes   = NULL;

	HRESULT hr = pUnk->QueryInterface(__uuidof(IBufferByteAccess), (void **)&pAccess);

	if (SUCCEEDED(hr))
	{
		hr = pAccess->Buffer(&bytes);
	
		if (SUCCEEDED(hr))
		{
			// Simple async task to update all the pixels to the average of RGB.
			//
			return create_async([=]()
			{
				auto length = buffer->Length;

				m_Effect.ApplygGrayscale(bytes, (int)buffer->Length);

				pAccess->Release();
			});
		}
		else
		{
			pAccess->Release();
			throw ref new Platform::Exception(hr, L"Couldn't get bytes from the buffer");
		}
	}
	else
		throw ref new Platform::Exception(hr, L"Couldn't access the buffer");
}

///////////////////////////////////////////////////////////////////////////////
//
//
Windows::Foundation::IAsyncAction^ ImageEffects::ApplyBrightnessScale(Windows::Storage::Streams::IBuffer^ buffer, int iBrightness)
{
	if (buffer == nullptr)
		throw ref new Platform::NullReferenceException(L"buffer cannot be null");

	// To access the buffer, we need to use old-style COM to get an interface to IBufferByteAccess
	//
	IUnknown*			pUnk    = reinterpret_cast<IUnknown*>(buffer);
	IBufferByteAccess*	pAccess = NULL;
	byte*				bytes   = NULL;

	HRESULT hr = pUnk->QueryInterface(__uuidof(IBufferByteAccess), (void **)&pAccess);

	if (SUCCEEDED(hr))
	{
		hr = pAccess->Buffer(&bytes);
	
		if (SUCCEEDED(hr))
		{
			// Simple async task to update all the pixels to the average of RGB.
			//
			return create_async([=]()
			{
				auto length = buffer->Length;

				m_Effect.ApplyBrightnessScale(bytes, (int)buffer->Length, iBrightness);

				pAccess->Release();
			});
		}
		else
		{
			pAccess->Release();
			throw ref new Platform::Exception(hr, L"Couldn't get bytes from the buffer");
		}
	}
	else
		throw ref new Platform::Exception(hr, L"Couldn't access the buffer");
}

///////////////////////////////////////////////////////////////////////////////
//
//
Platform::String^ ImageEffects::GetEngineVersion(String^ extension)
{
	auto	stdExtension = PlatformStringToCharArray(extension);
	auto	ansiVersion  = m_Effect.GetEngineVersion(stdExtension.get());

	return ref new String(CharArrayToPlatformString(ansiVersion).get());
}

///////////////////////////////////////////////////////////////////////////////
//
//
std::unique_ptr<char> ImageEffectsWP8::PlatformStringToCharArray(String^ input)
{
	auto   wideData   = input->Data();
	int    bufferSize = input->Length() + 1;
	char * ansi       = new char[bufferSize];

	WideCharToMultiByte(CP_UTF8, 0, wideData, -1, ansi, bufferSize, NULL, NULL);
	
	return std::unique_ptr<char>(ansi);
}

///////////////////////////////////////////////////////////////////////////////
//
//
std::unique_ptr<wchar_t> ImageEffectsWP8::CharArrayToPlatformString(char * input)
{
	int		  bufferSize  = strlen(input) + 1; 
	wchar_t * wideData    = new wchar_t[bufferSize];

	MultiByteToWideChar(CP_UTF8, 0, input, -1, wideData, bufferSize);
	
	return std::unique_ptr<wchar_t>(wideData);
}